import math
import numpy as np

L0=2*np.pi  # 1um
dx, dy = 0.01*L0, 0.01*L0  
ndx, ndy = 4000, 4000
Lx, Ly = ndx*dx, ndy*dy

# nc=1.1095e21 cm^-3
nU, nb = 42.27, 0.1
gamma = 100.
beta = math.sqrt(1.-1./gamma**2)
bx0, by0 = Lx-10*L0, 0.5*Ly
bwx0, bwy0 = 1.*L0, 0.8*L0
rtube, Rtube = 3*L0, 5*L0

Tsim = 200*L0
dt = 0.999/math.sqrt(1./ dx**2 + 1./ dy**2 )
diag_every = int(10*L0/dt)
ppc = 256


Main(
    geometry = "2Dcartesian",
    interpolation_order = 2 ,
    cell_length = [dx, dy],
    grid_length  = [Lx, Ly],
    number_of_patches = [32, 32],
    timestep = dt,
    simulation_time = Tsim,
    EM_boundary_conditions = [
                              ['silver-muller'],
                              ['reflective'],
                             ],
    solve_relativistic_poisson = True,
    solve_poisson = False,
    relativistic_poisson_max_iteration = 300000,
  #  reference_angular_frequency_SI = L0*3.e8/1.e-6,
    random_seed =  smilei_mpi_rank
)


# Initial plasma shape
def fbeam(x,y):
  if (x-bx0)**2<(3*bwx0)**2 and (y-by0)**2<(3*bwy0)**2:
    return nb*np.exp(-(x-bx0)**2/bwx0**2)*np.exp(-(y-by0)**2/bwy0**2)
  else:
    return 0.

def dens_U(x, y):
    if x>Lx and abs(y-0.5*Ly)>rtube and abs(y-0.5*Ly)<Rtube: return nU
    else: return 0.    
    
def dens_e(x, y):
    if x>Lx and abs(y-0.5*Ly)>rtube and abs(y-0.5*Ly)<Rtube: return 2*nU
    else: return 0.    
    
def dens_bondp(x, y):
    if y>(Ly-0.5*L0): return nU
    elif y<0.5*L0: return nU   
    else: return 0.
    
def dens_bonde(x, y):
    if y>(Ly-0.5*L0): return 2.*nU
    elif y<0.5*L0: return 2.*nU   
    else: return 0.


Species(
	name = 'bondp',
	position_initialization = 'random',
	momentum_initialization = 'cold',
	ionization_model = 'none',
	particles_per_cell = ppc,
    atomic_number = 92.,
	mass = 235.*1836.,
	charge = 2.,
	number_density = dens_bondp,
	mean_velocity = [0.,0.,0.],
	temperature = [0.,0.,0.],
	time_frozen = 0.,
	boundary_conditions = [
                           ['remove', 'remove'],
                           ['remove', 'remove']
                          ],
)


Species(
	name = 'bonde',
	position_initialization = 'random',
	momentum_initialization = 'cold',
	particles_per_cell = ppc,
	mass = 1.,
	charge = -1.,
	number_density = dens_bonde,
	mean_velocity = [0.,0.,0.],
	temperature = [0.,0.,0.],
	time_frozen = 0.,
	boundary_conditions = [
                           ['remove', 'remove'],
                           ['remove', 'remove']
                          ],
)


Species(
	name = 'Uranium',
	position_initialization = 'random',
	momentum_initialization = 'cold',
	ionization_model = 'none',
	particles_per_cell = ppc,
    atomic_number = 92.,
	mass = 235.*1836.,
	charge = 2.,
	number_density = dens_U,
	mean_velocity = [0.,0.,0.],
	temperature = [0.,0.,0.],
	time_frozen = 0.,
	boundary_conditions = [
                           ['remove', 'remove'],
                           ['remove', 'remove']
                          ],
)


Species(
	name = 'electron',
	position_initialization = 'random',
	momentum_initialization = 'cold',
	particles_per_cell = ppc,
	mass = 1.,
	charge = -1.,
	number_density = dens_e,
	mean_velocity = [0.,0.,0.],
	temperature = [0.,0.,0.],
	time_frozen = 0.,
	boundary_conditions = [
                           ['remove', 'remove'],
                           ['remove', 'remove']
                          ],
)


Species(
	name = 'Beam',
	position_initialization = 'random',
	momentum_initialization = 'cold',
	relativistic_field_initialization = True,
	particles_per_cell = ppc,
	c_part_max = 1.,
	mass = 1.,
	charge = -1.,
	number_density = fbeam,
	mean_velocity = [beta,0.,0.],
	temperature = [0.,0.,0.],
	time_frozen = 0.,
	boundary_conditions = [
                           ['remove', 'remove'],
                           ['remove', 'remove']
                          ],
)

MovingWindow(
    time_start = 0.,
    velocity_x = beta,
    number_of_additional_shifts = 0.,
    additional_shifts_time = 0.,
)


# Collisions(
    # species1 = ["electron"],
    # species2 = ["Uranium"],
    # coulomb_log = 0.,
    # ionizing = True
# )

# Collisions(
    # species1 = ["Beam"],
    # species2 = ["Uranium"],
    # coulomb_log = 0.,
# )

# Collisions(
    # species1 = ["Beam"],
    # species2 = ["electron"],
    # coulomb_log = 0.,
# )


DiagParticleBinning(
	deposited_quantity = "weight",
	every = diag_every,
	time_average = 1,
	species = ["electron"],
	axes = [
		 ["x",  0,  Lx, ndx+1],
         ["y",  0,  Ly, ndy+1] ]
)

DiagParticleBinning(
	deposited_quantity = "weight_ekin",
	every = diag_every,
	time_average = 1,
	species = ["electron"],
	axes = [
		 ["x",  0,  Lx, ndx+1],
         ["y",  0,  Ly, ndy+1] ]
)

DiagParticleBinning(
	deposited_quantity = "weight_ekin",
	every = diag_every,
	time_average = 1,
	species = ["Beam"],
	axes = [
		 ["x",  0,  Lx, ndx+1],
         ["y",  0,  Ly, ndy+1] ]
)

DiagParticleBinning(
	deposited_quantity = "weight_ekin",
	every = diag_every,
	time_average = 1,
	species = ["Uranium"],
	axes = [
		 ["x",  0,  Lx, ndx+1],
         ["y",  0,  Ly, ndy+1] ]
)

# DiagParticleBinning(
    # deposited_quantity = "weight",
    # every = diag_every,
    # time_average = 1,
    # species = ["Uranium"],
    # axes = [ ["charge",    0,   20,   21] ]
# )


DiagFields(
    every = diag_every,
    fields = ['Ex','Ey','Ez','Bx','By','Bz']
)


DiagFields(
    every = diag_every,
    fields = ['Rho_electron','Rho_Uranium','Rho_Beam']
)


DiagFields(
    every = diag_every,
    fields = ['Jx_electron','Jx','Jy','Jz']
)